#ifndef _IMGVOXEL_H
#define _IMGVOXEL_H


#include "IMGBBox.h"

namespace IMG {

#define VOXEL_FACE_LEFT   0
#define VOXEL_FACE_RIGHT  1
#define VOXEL_FACE_BEHIND 2
#define VOXEL_FACE_FRONT  3
#define VOXEL_FACE_UNDER  4
#define VOXEL_FACE_OVER   5
#define VOXEL_NO_FACE	  -1

//*
//	Voxel
//*
class Voxel : public BBox {
  public:
    //Voxel(Ogre::AxisAlignedBox &box);
    Voxel(BBox & box);

    Voxel();

    Voxel(const Voxel & v);

    inline ~Voxel() {};

    //the real voxel's id from scene, for indexing purpose
    inline unsigned int GetIdVoxel() const { return idvoxel;                };

    //return the number of polygons in the voxel
    // Gametools -- Isma 17/08/2005
    //int GetSizePolygonList()       const { return polygonlistid.GetSize(); }
    //int GetSizePatchList()       const { return patchlistid.GetSize(); }
    //return pointer to list_id_obspoly;
    //std::vector<unsigned int> * GetIdObsPolyList() { return &list_id_obspoly; }
    //bool InsertPolygon();
    //set the boundbox of the voxel
    //inline void SetBoundBox(Ogre::AxisAlignedBox *b)
    inline void SetBoundBox(BBox * b) {
        BBox::StartBoundingBox();
        mMinimum = b->getMinimum();
        mMaximum = b->getMaximum();
      };

    inline void SetBoundBox(float x1, float y1, float z1, float x2, float y2, float z2) {
        BBox::StartBoundingBox();
        BBox::AddBoundingVector3(x1, y1, z1);
        BBox::AddBoundingVector3(x2, y2, z2);
      };

    void Print();

    //print voxel info
	inline Ogre::Vector3 * GetVector3(int i) {
        	return BBox::GetCorner(i);
      };

    // Gametools -- Isma 17/08/2005
    //unsigned int GetNextPatch()	//return the id to next patch in the voxel
    //{
    //  if (index < GetSizePatchList()) return patchlistid.Get(index++);
    //  //return -1;
    //  return 0;
    //}
    // Gametools -- Isma 17/08/2005
    //unsigned int GetPatch(unsigned int i) { if (i < GetSizePatchList()) return patchlistid.Get(i); else return 0;}
    // Gametools -- Isma 17/08/2005
    //unsigned int GetPolygon(unsigned int i) { if (i < GetSizePolygonList())return polygonlistid.Get(i); else return 0; }
    // Gametools -- Isma 17/08/2005
    //unsigned int GetNextObsPoly() 	//return the id to next obspoly in the voxel
    //{
    //  if (index_obspoly < GetSizePolygonList()) return polygonlistid.Get(index_obspoly++);
    //  //return -1;
    //  return 0;
    //}
    bool AddPatch(unsigned int id);

    //add one patch (only the id) to list_id_patch
    bool AddPoly(unsigned int id);

    //add one patch (only the id) to list_id_obspoly
    //reset the index of the patch list
    inline void ResetIndex() { index = 0; };

    //reset the index of the obspolylist
    inline void ResetIndexObsPoly() { index_obspoly = 0; };


  private:
    static unsigned int ID_VOXELS;

    unsigned int idvoxel;

    //for GetNextPatch()
    unsigned int index;

    //for GetNextObsPoly	
    unsigned int index_obspoly;

    std::vector<unsigned int> list_id_obspoly;

    std::vector<unsigned int > list_id_patch;

};

}
#endif
